@@ -1,6 +1,6 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2009 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2011 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
 /* NXAGENT, NX protocol compression and NX extensions to this software    */
 /* are copyright of NoMachine. Redistribution and use of the present      */
@@ -135,6 +135,8 @@ char *nxagentKeyboard = NULL;
 
 Bool nxagentOnce = True;
 
+int nxagentRemoteMajor = -1;
+
 static void nxagentParseOptionString(char*);
 
 /*
@@ -512,6 +514,22 @@ int ddxProcessArgument(int argc, char *argv[], int i)
     return 0;
   }
 
+  if (!strcmp(argv[i], "-irlimit"))
+  {
+    int limit;
+
+    if (++i < argc &&
+            sscanf(argv[i], "%i", &limit) == 1)
+    {
+      nxagentChangeOption(ImageRateLimit, limit);
+
+  
+      return 2;
+    }
+
+    return 0;
+  }
+
   if (!strcmp(argv[i], "-tile"))
   {
     int width;
@@ -562,11 +580,15 @@ int ddxProcessArgument(int argc, char *argv[], int i)
       if (!strcmp(argv[i],"fullscreen"))
       {
         nxagentChangeOption(Fullscreen, True);
+
+        nxagentChangeOption(AllScreens, True);
       }
       else if (!strcmp(argv[i],"ipaq"))
       {
         nxagentChangeOption(Fullscreen, True);
 
+        nxagentChangeOption(AllScreens, True);
+
         nxagentIpaq = True;
       }
       else
@@ -774,6 +796,8 @@ int ddxProcessArgument(int argc, char *argv[], int i)
   {
     nxagentChangeOption(DeviceControl, True);
 
+    nxagentChangeOption(DeviceControlUserDefined , True);
+
     return 1;
   }
 
@@ -1071,10 +1095,14 @@ static void nxagentParseOptions(char *name, char *value)
     else if (!strcmp(value, "1"))
     {
       nxagentChangeOption(Fullscreen, True);
+
+      nxagentChangeOption(AllScreens, True);
     }
     else if (!strcmp(value, "0"))
     {
       nxagentChangeOption(Fullscreen, False);
+
+      nxagentChangeOption(AllScreens, False);
     }
     else
     {
@@ -1208,7 +1236,7 @@ static void nxagentParseOptions(char *name, char *value)
     {
       nxagentChangeOption(ClientOs, ClientOsSolaris);
     }
-    else if (strcmp(value, "mac") == 0)
+    else if (strcmp(value, "macosx") == 0)
     {
       nxagentChangeOption(ClientOs, ClientOsMac);
     }
@@ -1512,7 +1540,7 @@ N/A
       int splitMode = 0;
       int splitSize = 0;
 
-      unsigned int packMethod  = PACK_NONE;
+      unsigned int packMethod = PACK_NONE;
       unsigned int packQuality = 9;
 
       int dataLevel   = 0;
@@ -1570,6 +1598,8 @@ N/A
         nxagentAlphaCompat = 0;
       }
 
+      nxagentRemoteMajor = remoteMajor;
+
       if (nxagentPackMethod == -1)
       {
         nxagentPackMethod = packMethod;
@@ -1680,6 +1710,11 @@ N/A
       nxagentChangeOption(Fullscreen, False);
     }
 
+    if (nxagentOption(AllScreens) == UNDEFINED)
+    {
+      nxagentChangeOption(AllScreens, False);
+    }
+
     if (nxagentOption(Binder) == UNDEFINED)
     {
       nxagentChangeOption(Binder, False);
