@@ -1,6 +1,6 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2009 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2011 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
 /* NXAGENT, NX protocol compression and NX extensions to this software    */
 /* are copyright of NoMachine. Redistribution and use of the present      */
@@ -159,6 +159,8 @@ struct _DispatchRec nxagentDispatch = { UNDEFINED, 0, 0, 0 };
  * for our clients or the X server.
  */
 
+extern int nxagentSkipImage;
+
 void nxagentBlockHandler(pointer data, struct timeval **timeout, pointer mask)
 {
   /*
@@ -357,7 +359,7 @@ void nxagentBlockHandler(pointer data, struct timeval **timeout, pointer mask)
                            nxagentCorruptedBackgrounds > 0 ||
                                nxagentCorruptedPixmaps > 0));
 
-    if (synchronize == 1)
+    if (nxagentSkipImage == 0 && synchronize == 1)
     {
       #ifdef TEST
       fprintf(stderr, "nxagentBlockHandler: Setting a zero timeout with [%d][%d][%d] and "
@@ -527,7 +529,8 @@ void nxagentBlockHandler(pointer data, struct timeval **timeout, pointer mask)
                 synchronize, nxagentReady);
     #endif
 
-    if (nxagentQueuedEvents(nxagentDisplay) > 0)
+    if (NXDisplayError(nxagentDisplay) == 0 &&
+            nxagentQueuedEvents(nxagentDisplay) > 0)
     {
       #ifdef WARNING
       fprintf(stderr, "nxagentBlockHandler: WARNING! Forcing a null timeout with events queued.\n");
