@@ -1,6 +1,6 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2009 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2011 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
 /* NXAGENT, NX protocol compression and NX extensions to this software    */
 /* are copyright of NoMachine. Redistribution and use of the present      */
@@ -63,6 +63,9 @@ void nxagentInitOptions()
   nxagentOptions.Height      = 0;
   nxagentOptions.BorderWidth = 0;
 
+  nxagentOptions.WMBorderWidth = -1;
+  nxagentOptions.WMTitleHeight = -1;
+
   nxagentOptions.SavedX      = 0;
   nxagentOptions.SavedY      = 0;
   nxagentOptions.SavedWidth  = 0;
@@ -82,6 +85,8 @@ void nxagentInitOptions()
 
   nxagentOptions.DeviceControl = 0;
 
+  nxagentOptions.DeviceControlUserDefined = 0;
+
   nxagentOptions.ResetKeyboardAtResume = 1;
 
   nxagentOptions.Reset = 0;
@@ -146,6 +151,8 @@ void nxagentInitOptions()
   nxagentOptions.InhibitXkb = 1;
 
   nxagentOptions.CopyBufferSize = COPY_UNLIMITED;
+
+  nxagentOptions.ImageRateLimit = 0;
 }
 
 /*
@@ -168,6 +175,9 @@ void nxagentResetOptions()
 
   nxagentOptions.TileWidth  = UNDEFINED;
   nxagentOptions.TileHeight = UNDEFINED;
+
+  nxagentOptions.WMBorderWidth = -1;
+  nxagentOptions.WMTitleHeight = -1;
 }
 
 void nxagentSaveOptions()
