@@ -1,6 +1,6 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2009 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2011 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
 /* NXAGENT, NX protocol compression and NX extensions to this software    */
 /* are copyright of NoMachine. Redistribution and use of the present      */
@@ -102,6 +102,13 @@ typedef struct _AgentOptions
 
   int Fullscreen;
 
+  /*
+   * True if the fullscreen NX session will
+   * extend on all available screens.
+   */
+
+  int AllScreens;
+
   /*
    * Set to the auto-disconnect timeout, if
    * the user activated this feature.
@@ -126,6 +133,9 @@ typedef struct _AgentOptions
    * screen.
    */
 
+  int WMBorderWidth;
+  int WMTitleHeight;
+
   int SavedX;
   int SavedY;
   int SavedWidth;
@@ -173,6 +183,12 @@ typedef struct _AgentOptions
 
   int DeviceControl;
 
+  /*
+   * Explicitly asked config propagation.
+   */
+
+  int DeviceControlUserDefined;
+
   /*
    * Resuming keyboard device corrects keymap if session
    * migrates across platforms with different keycode
@@ -359,6 +375,12 @@ typedef struct _AgentOptions
 
   int CopyBufferSize;
 
+  /*
+   * Max image data rate to the encoder input.
+   */
+
+  int ImageRateLimit;
+
 } AgentOptionsRec;
 
 typedef AgentOptionsRec *AgentOptionsPtr;
